/** Shadowed Horizons Software Development Kit:
*    The MetaObject class is copyright(c) 2004 smtose.org
*
*    This class is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    This class is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
**/
package org.smtose.shadowedhorizons.metaobject;
import java.util.HashMap;
import java.util.Iterator;
import org.smtose.shadowedhorizons.environment.*;


/**
 * Shadowed Horizons Software Development Kit: <br>
 * A <code>MetaObject</code> is a base object that provides a number
 * of generic methods that most objects created using this kit will
 * need.
 *
 * <p> The <code>MetaObject</code> class is
 * copyright&copy; 2004 smtose.org (www.smtose.org)
 *
 * @author Gregory Brown
 * @author smtose.org
 **/

public abstract class MetaObject implements Locatable {
	private String name;
	private String description;
	private HashMap booleanAttributes;
        private	HashMap integerAttributes;
       	private HashMap stringAttributes;
	private HashMap inventory;
	private Location loc;
	private Environment env;
	
	private void init() {
		this.name = "";
		this.description = "";
		this.booleanAttributes = new HashMap();
		this.integerAttributes = new HashMap();
		this.stringAttributes = new HashMap();
		this.inventory = new HashMap();
	}

	/** Constructs an empty <code>MetaObject</code>
	 **/ 	
	public MetaObject() {
		init();
	}
	/** Constructs a named <code>MetaObject</code>
	 *  @param name	The name the <code>MetaObject</code> will receive
	 **/
	
	public MetaObject(String name) {
		init();
		this.name = name;
	}
	
	/** Constructs a named <code>MetaObject</code> with a description
	 *  @param name	The name the <code>MetaObject</code> will receive
	 *  @param description	The description the <code>MetaObject</code> will receive
	 **/
	public MetaObject(String name, String description) {
		init();
		this.name = name;
		this.description = description;
	}
	/** Constructs a fully structured <code>MetaObject</code> (Intended to be
	 *  used within another constructor)
	 *  @param name The name the <code>MetaObject</code> will receive
	 *  @param description The description the <code>MetaObject</code> will recieve
	 *  @param booleanAttributes The boolean attributes of this <code>MetaObject</code>
	 *  @param integerAttributes The integer attributes of this <code>MetaObject</code>
	 *  @param stringAttributes  The string attributes of this <code>MetaObject</code>
	 *  @param inventory	     A hashmap of objects to be added to the Inventory
	 **/
	public MetaObject(String name, String description, HashMap booleanAttributes,
		          HashMap integerAttributes, HashMap stringAttributes, HashMap inventory) {
       		this.name = name;
		this.description = description;
		this.booleanAttributes = booleanAttributes;
		this.integerAttributes = integerAttributes;
		this.stringAttributes = stringAttributes;
		this.inventory = inventory;
		this.env = null;
	}
	/** Returns the <code>MetaObject</code>'s name
	 *  @return	The name of the MetaObject
	 **/
	public String name() {
		return this.name;
	}
	/** Returns the <code>MetaObject</code>'s description
	 *  @return	The description of the MetaObject
	 **/
	public String description() {
		return this.description;
	}
	
	/** Gets all the items in the inventory
	 *  @return	The inventory HashMap
	 **/
	public HashMap held() {
		return this.inventory;
	}
	
	/** Gets the state of a boolean property
	 *  @param key	the name of the property
	 *  @return	the boolean state of the property (true/false)
	 **/
	
	public boolean booleanValue(String key) {
		return ( (Boolean) this.booleanAttributes.get(key) ).booleanValue();
	}

	/** Sets the state of a boolean property
	 *  @param key  the name of the property
	 *  @param value the boolean state to set the property to
	 **/
	public void booleanValue(String key, boolean value) {
		Boolean objectValue = new Boolean(value);
		this.booleanAttributes.put(key,objectValue);
	}
        
	/** Sets the value of an integer property
	 *  @param key the name of the property
	 *  @param value the integer value to set the property to
	 **/
        public void integerValue(String key, int value) {
                Integer objectValue = new Integer(value);
                this.integerAttributes.put(key,objectValue);
        }
	
	/** Gets the value of an integer property
	 *  @param key the name of the property`
	 **/
	
        public int integerValue(String key) {
                return ( (Integer) this.integerAttributes.get(key) ).intValue();
        }

	/** Deincrements the value of an integer property's value by 1
	 * @param key the name of the property
	 **/
	public void deInc(String key) {
		this.integerValue(key, this.integerValue(key) - 1);
	}

	/** Increments the value of an integer property's value by 1
	 * @param key the name of the property
	 **/
	public void Inc(String key) {
		this.integerValue(key, this.integerValue(key) + 1);
	}

	/** Adds <code>value</code> to an integer property's value
	 * @param key the name of the property
	 * @param value the value to be added to the property's current value
	 **/
	public void plusEquals(String key, int value) {
		this.integerValue(key, this.integerValue(key) + value);
	}

	/** Subtracts <code>value</code> from an integer property's value
	 * @param key the name of the property
	 * @param value the value to be subtracted from the property's current value
	 **/
	public void minusEquals(String key, int value) {
		this.integerValue(key, this.integerValue(key) - value);
	}

	/** Multiplies an integer property's value by a specified <code>value</code>
	 * @param key the name of the property
	 * @param value the value to multiply the property's value by
	 **/
	public void timesEquals(String key, int value) {
		this.integerValue(key, this.integerValue(key) * value);
	}

	/** Inverts the state of a boolean value
	 * @param key the name of the property
	 **/
	
	public void invert(String key) {
		this.booleanValue(key, !this.booleanValue(key));
	}
	
	/** Returns the value of a String property
	 * @param key the name of the property
	 * @return The String value of a property
	 **/	
	public String stringValue(String key) {
		return ( (String) this.stringAttributes.get(key) );
	
	}
	
	/** Sets the value of a String property
	 * @param key the name of the property
	 * @param value the String value to set the property to
	 **/
	public void stringValue(String key, String value){
		this.stringAttributes.put(key,value);
	}

	/** Adds an item to the <code>MetaObject</code>'s Inventory
	 * @param key the name of the property
	 * @param anObject the object to be added to the inventory
	 **/
        public void addInventoryItem(String key, Object anObject){
                this.inventory.put(key,anObject);
        }

	/** Checks to see if a slot is empty in the <code>MetaObject's</code> inventory
	 * @param key the name of the slot to check
	 * @return the state of the slot checked
	 **/
        public boolean isEmpty(String key) {
            if (this.inventory.get(key) != null)
                return false;
            else
                return true;
        }

	/** Gets an object from inventory
	 * @param key the name of the slot to get the object from
	 * @return The requested inventory object
	 **/
        public Object getInventoryItem(String key) {
            if (this.isEmpty(key)) 
                return null;
            else {
                Object anObject = this.inventory.get(key);
                return anObject;
            }
        }
        /** Gets an object from inventory, optionally removing the object from the inventory list
	 * @param key the name of the slot to get the object from
	 * @param removeAfterGet set this to true if you want object to be automatically removed
	 * @return The requested inventory object
	 **/
	public Object getInventoryItem(String key, boolean removeAfterGet) {
		if (this.isEmpty(key))
			return null;
		else {
			Object anObject = this.inventory.get(key);
			if (removeAfterGet)
				this.inventory.remove(key);
			return anObject;
		}
	}
	
	/** Sets the <code>MetaObject</code>'s Location in (row, column) form
	 * @param row The row to set the <code>MetaObject</code> at
	 * @param col The column to set the <code>MetaObject</code> at
	 **/
	public void setLocation(int row, int col) {
		this.loc = new Location(row,col);
	}
	
	/** Sets the <code>MetaObject</code>'s location in (x,y) form
	 * @param x The x coordinate to set the <code>MetaObject</code> at
	 * @param y The y coordinate to set the <code>MetaObject</code> at
	 **/
	public void setGeometricLocation(int x, int y) {
		this.loc = new Location(y, x);
	}

	/** Sets the <code>MetaObject</code>'s location to the specified location
	 * @param loc The location to set the <code>MetaObject</code> to
	 **/
	public void setLocation(Location loc) {
		this.loc = loc;
	}
	
	/** Gets the <code>MetaObject</code>'s Location
	 * @return A <code>Location</code> object 
	 **/
	public Location getLocation() {
		return this.loc;
	}
	
	/** Gets the <code>MetaObject</code>'s Environment
	 *  @return The <code>Environment</code>
	 **/
	public Environment getEnvironment() {
		return this.env;
	}
	
	/** Sets the <code>MetaObject</code>'s Environment
	 * @param newEnv The new <code>Environment</code>
	 * @param newLoc The new <code>Location</code>
	 **/
	
	public void setEnvironment(Environment newEnv, Location newLoc) {
		if (this.env != null)
			this.env.remove(this);
		this.setLocation(newLoc);
		this.env = newEnv;
		this.env.add(this);
	}	
	/** The action to be performed (must be overridden)
	 **/
	public abstract void act();

	/** Locates and returns an object in the <code> MetaObject's inventory (Thanks to James Edward Gray II)
	 * @param key The item's identifier
	 * @return the <code>MetaObject</code> requested
	 **/
	public MetaObject locate(String key) {					
		if (this.held().containsKey(key))
			return (MetaObject) this.held().get(key);
		else if (this.held() != null) {
			for (Iterator i = this.held().values().iterator(); i.hasNext();) {
				MetaObject found = ((MetaObject) i.next() ).locate(key);
				if (found != null) return found;
			}
		}
	       	return null; 
	}



	 
}
