// Shadowed Horizons Software Development Kit:
// // This implementation of John Conway's Game Of Life is copyright(c) 2004 smtose.org
// // 
// // This class is free software; you can redistribute it and/or modify
// // it under the terms of the GNU General Public License as published by
// // the Free Software Foundation.
// //
// // This class is distributed in the hope that it will be useful,
// // but WITHOUT ANY WARRANTY; without even the implied warranty of
// // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// // GNU General Public License for more details.
package org.smtose.shadowedhorizons.test;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.smtose.shadowedhorizons.environment.*;
import org.smtose.shadowedhorizons.metaobject.*;
import java.util.ArrayList;
public class Life {
	public static final int NUM_ROWS = 50;
	public static final int NUM_COLS = 50;
	public static final int SQUARE_DIM = 10;
	public static void main(String[] args) {
			JFrame frame = new JFrame( "Conway's Game of Life" );
			JPanel panel = new JPanel();
			BoundedEnv env = initEnv(new BoundedEnv(NUM_ROWS,NUM_COLS));
			EnvDisplay display = new EnvDisplay(env, SQUARE_DIM);
			JButton button = new JButton("Step");
			JButton button2 = new JButton("Clear");
			JButton button3 = new JButton("Start/Stop");
			ButtonHandler handler = new ButtonHandler(env,display);	
			button.addActionListener(handler);
			button2.addActionListener(handler);
			button3.addActionListener(handler);
			display.setSize( SQUARE_DIM * (env.numCols() + 1), SQUARE_DIM * (env.numRows() + 1) );
			panel.add(display);
			panel.add(button);
			panel.add(button2);
			panel.add(button3);
			frame.getContentPane().add(panel);
			frame.setSize(10*(env.numCols() * SQUARE_DIM)/9,(env.numRows() * SQUARE_DIM)+100);
			frame.setVisible( true );			
	}
	static BoundedEnv initEnv(BoundedEnv env){
		for (int row = 0; row < env.numRows(); row++) {
			for (int col = 0; col < env.numCols(); col++) {
				env.add(new Cell(new Location(row,col),env));
			}
		}
		return env;
	}
}
class ButtonHandler implements ActionListener {
	Environment env;
	EnvDisplay display;
	SimulationController taskPerformer;
	int delay;
	Timer aTimer;
	boolean isRunning = false;

	public ButtonHandler(Environment env, EnvDisplay display) {	
		this.env = env;
		this.display = display;
 		this.delay = 100; //milliseconds
		this.taskPerformer = new SimulationController(this.env, this.display);
  	
  		this.aTimer = new Timer(delay, taskPerformer);
	}
	public void actionPerformed(ActionEvent event) {
		String command = event.getActionCommand();
		//If the function list were bigger, a simulation class would make this cleaner
		
		if (isRunning && command == "Start/Stop") {
			aTimer.stop();
			isRunning = false;
			return;
		}
		else if ( command == "Start/Stop" ) {
			aTimer.start();
			isRunning = true;
			return;

		}
		for (int row = 0; row < this.env.numRows(); row++) {
			for (int col = 0; col < this.env.numCols(); col++ ) {
				( (MetaObject) env.objectAt(new Location(row,col)) ).booleanValue("Last_State",
				  ( (MetaObject) env.objectAt(new Location(row,col)) ).booleanValue("Alive"));
			}
		}
		
		for (int row = 0; row < env.numRows(); row++) {
			for (int col = 0; col < env.numCols(); col++) {
				if (command == "Step") { 
					( (MetaObject) env.objectAt(new Location(row,col)) ).act();
				}
				else if (command == "Clear")
					( (MetaObject) env.objectAt(new Location(row,col)) ).booleanValue("Alive",false);
				
			}
		}
		display.repaint();
	}
	
}

class SimulationController implements ActionListener {
	Environment env;
	EnvDisplay display;
	public SimulationController (Environment env, EnvDisplay display) {
		this.env = env;
		this.display = display;
	}
	public void actionPerformed(ActionEvent evt) {
		for (int row = 0; row < this.env.numRows(); row++) {
			for (int col = 0; col < this.env.numCols(); col++ ) {
				( (MetaObject) env.objectAt(new Location(row,col)) ).booleanValue("Last_State",
				( (MetaObject) env.objectAt(new Location(row,col)) ).booleanValue("Alive"));
			}
		}
		for (int row = 0; row < this.env.numRows(); row++) {
			for (int col = 0; col < this.env.numCols(); col++) {
				( (MetaObject) env.objectAt(new Location(row,col)) ).act();
	       	        }
		}
		display.repaint();
		
	}

}
class EnvDisplay extends Canvas implements MouseMotionListener {
	//not generic, but doesn't need to be, dealing with homogenous object types 
	BoundedEnv env;
	int squareDim;
	
	public EnvDisplay( BoundedEnv env, int squareDim ) {
	       	this.env = env;
 		addMouseMotionListener( this );
		this.squareDim = squareDim;
		
 	}
		
	public void mouseDragged( MouseEvent event ) {
   		int rowClicked = event.getY() / squareDim;
		int colClicked = event.getX() / squareDim;
		Location locationClicked = new Location(rowClicked,colClicked);
		if ( event.isMetaDown() )
			((Cell)env.objectAt(locationClicked)).booleanValue("Alive",false);
		else {
			((Cell)env.objectAt(locationClicked)).booleanValue("Alive",true);
			((Cell)env.objectAt(locationClicked)).booleanValue("Last_State",true);
		}
		repaint(); 
	}
	
	public void mouseMoved ( MouseEvent event ) { }
	public void paint(Graphics g) {
		for (int row = 0; row < env.numRows(); row++) {
			for (int col = 0; col < env.numCols(); col++) {
				Cell c = (Cell) env.objectAt(new Location(row,col));
				if ( c != null && c.booleanValue("Alive") ) {
					g.setColor(c.color());
					g.fillRect( col * squareDim, row * squareDim, squareDim, squareDim);
				}
				else {
					g.setColor(Color.BLACK);		
					g.drawRect(col * squareDim,  row * squareDim, squareDim, squareDim);
			
				}
			}
		}
	}



	
}
	
class Cell extends MetaObject implements Locatable {
	Environment env;
	Color color;	
	RandNumGenerator randomNumGenerator = new RandNumGenerator();
	public Cell(Location loc, Environment env) {
		super();
		this.setLocation(loc);
		this.env = env;
		this.booleanValue("Alive",false);
		this.booleanValue("Last_State",false);
		this.color = randomColor();
	}
	public void act() {
		ArrayList neighbors = env.neighborsOf(this.getLocation());
		int countLives = 0;
		for (int i = 0; i < neighbors.size(); i++) {
			if ( ( (MetaObject) (env.objectAt( (Location) neighbors.get(i) ) ) ).booleanValue("Last_State") )
				       countLives++;	
		}
		if ( this.booleanValue("Last_State") && (countLives < 2 || countLives > 3) ) {this.booleanValue("Alive",false); } //Death
		if ( this.booleanValue("Last_State") && (countLives == 2 || countLives == 3) ) { } //Survival
		if ( !this.booleanValue("Last_State") && countLives == 3) { this.booleanValue("Alive",true); } //Birth

	}

	public void interact() {
		this.booleanValue("Alive",!this.booleanValue("Alive"));
	}
	public Color randomColor() {
		return new Color( randomNumGenerator.getInstance().nextInt(255),
				  randomNumGenerator.getInstance().nextInt(255),
				  randomNumGenerator.getInstance().nextInt(255)
				);
	}

	public Color color() {
		return this.color;
	}
}

