/**
 * Shadowed Horizons Software Development Kit:
 * 
 * The Pong Test Class is copyright(c) 2004 smtose.org
 * 
 * This class is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation.
 * 
 * This class is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 */
package org.smtose.shadowedhorizons.test;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;

import org.smtose.shadowedhorizons.environment.RandNumGenerator;
import org.smtose.shadowedhorizons.metaobject.MetaObject;

public class Pong {
	public static void main(String[] args) {
		PongGUI pInter = new PongGUI();
	}
}

class PongGUI implements KeyListener {
	JFrame frame;
	JPanel panel;
	ActionArea actionArea;

	public PongGUI() {
		this.frame = new JFrame("Pong");
		this.panel = new JPanel();
		this.actionArea = new ActionArea();
		panel.add(actionArea);
		frame.getContentPane().add(panel);

		frame.addKeyListener(this);
		frame.setSize(actionArea.getWidth(), actionArea.getHeight());
		frame.setVisible(true);
		frame.setDefaultCloseOperation(frame.EXIT_ON_CLOSE);
	}

	public void keyPressed(KeyEvent evt) {
		actionArea.draw(evt.getKeyChar());
	}

	public void keyReleased(KeyEvent evt) {
	}

	public void keyTyped(KeyEvent evt) {
	}

}

class ActionArea extends Canvas {
	int x = 50;
	int y = 50;
	String command = "";

	Paddle b = new Paddle(50, 50, 20, 50, 400);
	Paddle b2 = new Paddle(50, 500, 20, 50, 400);
	Ball ball = new Ball(50, 275, 20, 20, 600, 400, b, b2);

	Timer aTimer = new Timer(30, new Animator(ball, b2, this));

	public ActionArea() {
		super();
		this.setSize(600, 400);
		aTimer.start();
		b.setBall(ball);
		b2.setBall(ball);

	}

	public void paint(Graphics g) {
		if (command.equals("win")) {
			g.drawString("You Win", 300, 200);
		}
		else if (command.equals("lose"))
			g.drawString("You Lose", 300, 200);
		else {
			g.fillRect(b.getLocation().col(), b.getLocation().row(), b.width(), b.height());
			g.fillRect(b2.getLocation().col(), b2.getLocation().row(), b2.width(), b2.height());
			g.setColor(Color.RED);
			g.fillRect(b.getLocation().col() + 2, b.getLocation().row() + 2, b.width() - 4, b.height() - 4);
			g.setColor(Color.BLUE);
			g.fillRect(b2.getLocation().col() + 2, b2.getLocation().row() + 2, b2.width() - 4, b.height() - 4);
			g.setColor(Color.BLACK);
			g.fillOval(ball.getLocation().col(), ball.getLocation().row(), ball.width(), ball.height());
			g.setColor(Color.GREEN);
			g.fillOval(ball.getLocation().col() + 2, ball.getLocation().row() + 2, ball.width() - 4, ball.height() - 4);

		}

	}

	public void draw(char keyPressed) {
		int paddleX = b.getLocation().col();
		int paddleY = b.getLocation().row();
		if (b.getLocation().row() - 8 >= 0 && keyPressed == 'w')
			b.setLocation(paddleY - 8, paddleX);
		if (b.getLocation().row() + 3 * b.height() / 2 + 8 <= this.getHeight() && keyPressed == 's')
			b.setLocation(paddleY + 8, paddleX);
		repaint();
	}

	public void end() {
		aTimer.stop();
		if (ball.width() == 0)
			command = "lose";
		else if (ball.height() == 0)
			command = "win";
		repaint();
	}

}

class Animator implements ActionListener {
	MetaObject animatedObject;
	MetaObject animatedObject2;
	ActionArea a;

	public Animator(MetaObject animatedObject, MetaObject animatedObject2, ActionArea a) {
		this.animatedObject = animatedObject;
		this.animatedObject2 = animatedObject2;
		this.a = a;
	}

	public void actionPerformed(ActionEvent event) {
		this.animatedObject.act();
		if (((Ball) animatedObject).width() == 0 || ((Ball) animatedObject).height() == 0)
			a.end();
		else {
			if (animatedObject2.getLocation().col() - animatedObject.getLocation().col() < 300
		            && animatedObject2.getLocation().col() - animatedObject.getLocation().col() > 0)
				((Paddle) this.animatedObject2).track(true);
			else
				((Paddle) this.animatedObject2).track(false);
			a.repaint();
			animatedObject2.act();

		}

	}
}

class Paddle extends MetaObject {
	Ball ball;

	public Paddle(int row, int col, int w, int h, int maxY) {
		this.setLocation(row, col);
		this.integerValue("w", w);
		this.integerValue("h", h);
		this.integerValue("movesLeft", 0);
		this.integerValue("maxY", maxY);
		this.booleanValue("goUp", false);
		this.booleanValue("tracking", false);
		this.ball = null;
	}

	public int width() {
		return this.integerValue("w");
	}

	public int height() {
		return this.integerValue("h");
	}

	public void setBall(Ball ball) {
		this.ball = ball;
	}

	public void act() {
		if (this.booleanValue("tracking")) {
			if (this.getLocation().row() > this.ball.getLocation().row() && this.getLocation().row() - 5 >= 0)
				this.setLocation(this.getLocation().row() - 5, this.getLocation().col());
			else if (this.getLocation().row() + 3 * this.height() / 2 + 5 <= this.integerValue("maxY"))
				this.setLocation(this.getLocation().row() + 5, this.getLocation().col());
		}

		else if (this.booleanValue("goUp") && (this.getLocation().row() - 5 >= 0))
			this.setLocation(this.getLocation().row() - 5, this.getLocation().col());
		else if (this.getLocation().row() + 3 * this.height() / 2 + 5 <= this.integerValue("maxY"))
			this.setLocation(this.getLocation().row() + 5, this.getLocation().col());
		else
			this.integerValue("movesLeft", 0);

		if (this.integerValue("movesLeft") == 0)
			randomizer();

		this.deInc("movesLeft");
	}

	public void track(boolean state) {
		this.booleanValue("tracking", state);
	}

	public void randomizer() {
		this.integerValue("movesLeft", RandNumGenerator.getInstance().nextInt(8) + 2);
		this.booleanValue("goUp", RandNumGenerator.getInstance().nextBoolean());
	}
}

class Ball extends MetaObject {
	private int w;
	private int h;
	private int maxX;
	private int maxY;
	private int dy = 1;
	private int dx = 1;
	private int pitchX;
	private int pitchY;
	Paddle b;
	Paddle b2;

	public Ball(int row, int col, int w, int h, int maxX, int maxY, Paddle b, Paddle b2) {
		this.setLocation(row, col);
		this.w = w;
		this.h = h;
		this.maxX = maxX;
		this.maxY = maxY;
		this.pitchX = RandNumGenerator.getInstance().nextInt(3) + 4;
		this.pitchY = RandNumGenerator.getInstance().nextInt(3) + 4;
		this.b = b;
		this.b2 = b2;
	}

	public void act() {
		if (inBounds() == 1) {
			this.w = 0;
		}

		else if (inBounds() == 2) {
			this.h = 0;
		}

		else {

			this.setLocation(this.getLocation().row() + pitchX * (dy), this.getLocation().col() + pitchY * (dx));

			if (this.getLocation().row() <= 0 || this.getLocation().row() + this.h + 30 >= maxY) {
				dy *= -1;
				randomize();
			}
			else if (collision() || this.getLocation().col() <= 30 || this.getLocation().col() + this.w + 30 >= maxX) {
				dx *= -1;
				randomize();
			}
		}
	}

	public int width() {
		return this.w;
	}

	public int height() {
		return this.h;
	}

	public void randomize() {
		this.pitchX = RandNumGenerator.getInstance().nextInt(3) + 2;
		this.pitchY = RandNumGenerator.getInstance().nextInt(3) + 2;
	}

	public boolean collision() {
		if ((this.getLocation().col() >= 68 && this.getLocation().col() <= 75) && (this.getLocation().row() + this.height() >= b.getLocation().row() && this.getLocation().row() <= b.getLocation().row() + 50) || (this.getLocation().col() + this.width() >= 498 && this.getLocation().col() + this.width() <= 505) && (this.getLocation().row() + this.height() >= b2.getLocation().row() && this.getLocation().row() <= b2.getLocation().row() + 50))

			return true;
		return false;
	}

	public int inBounds() {
		if (this.getLocation().col() <= 50)
			return 1;
		else if (this.getLocation().col() >= 520)
			return 2;
		else
			return 0;
	}
}
