// Shadowed Horizons Software Development Kit:
// // The Portal class is copyright(c) 2004 smtose.org
// //
// // This class is free software; you can redistribute it and/or modify
// // it under the terms of the GNU General Public License as published by
// // the Free Software Foundation.
// //
// // This class is distributed in the hope that it will be useful,
// // but WITHOUT ANY WARRANTY; without even the implied warranty of
// // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// // GNU General Public License for more details.
package org.smtose.shadowedhorizons.world;
import org.smtose.shadowedhorizons.environment.*;
import org.smtose.shadowedhorizons.metaobject.*;

/**
 * Shadowed Horizons Software Development Kit: <br>
 * A <code>Portal</code> object provides functionality
 * to link <code>Environment</code> objects together.
 *
 * <p> The <code>Portal</code> class is
 * copyright&copy; 2004 smtose.org (www.smtose.org)
 *
 * @author Gregory Brown
 * @author smtose.org
 **/


public class Portal extends MetaAdapter {
	private Environment destination;
	private Location namedPoint;
	
	/** Constructs a portal with a given destination
	 * @param destination The <code>Environment</code> the portal will point to
	 * @param namedPointIdentifier The identifier of the named point to use
	 **/
	public Portal(Environment destination, String namedPointIdentifier) {
		this.destination = destination;
		this.namedPoint = this.destination.getNamedPoint(namedPointIdentifier);
		this.booleanValue("Activated",true);	
		
	}
	
	/** Constructs a portal with a given destination and activation state
	 * @param destination The <code>Environment</code> the portal will point to
	 * @param namedPointIdentifier The identifier of the named point to use
	 * @param activated The status of activation
	 **/
	public Portal(Environment destination, String namedPointIdentifier, boolean activated) {
		this.destination = destination;
		this.namedPoint = this.destination.getNamedPoint(namedPointIdentifier);
		this.booleanValue("Activated", activated);
	}
	
	/** Checks to see if the <code>Portal</code> is activated
	 * @return The activation status of the <code>Portal</code>
	 **/
	public boolean isActivated() {
		if (this.booleanValue("Activated"))
			return true;
		else
			return false;
	}

	/** Activates the <code>Portal</code>
	 **/
	public void activate() {
		this.booleanValue("Activated", true);
	}

	/** Deactivates the <code>Portal</code>
	 **/
	public void disable() {
		this.booleanValue("Activated",true);
	}

	/** Gets the <code>Portal</code>'s destination
	 * @return The <code> Environment </code> this <code>Portal</code> points to.
	 **/
	public Environment destination() {
		return this.destination;
	}

	/** Moves an object from one environment to another
	 * @param anObject the object to move
	 **/
	public void interact(MetaObject anObject) {
		
		if (this.isActivated()) 
			anObject.setEnvironment(this.destination, this.namedPoint);
	}
}
		
