/** 
 * 
 * Shadowed Horizons Software Development Kit:
 * The World class is copyright(c) 2004 smtose.org
 * This class is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation.
 * 
 * This class is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */
package org.smtose.shadowedhorizons.world;

import org.smtose.shadowedhorizons.environment.Environment;
import org.smtose.shadowedhorizons.metaobject.MetaAdapter;
import org.smtose.shadowedhorizons.metaobject.MetaObject;

/**
 * Shadowed Horizons Software Development Kit: <br>
 * A <code>World</code> object provides a way to organize many <code>Environments</code>.
 * 
 * <p>
 * The <code>World</code> class is copyright&copy; 2004 smtose.org (www.smtose.org)
 * 
 * @author Gregory Brown
 * @author smtose.org
 */

public class World extends MetaAdapter {

	/**
	 * Constructs an empty <code>World</code> object
	 */
	public World() {
		super();
	}

	/**
	 * Constructs a <code>World</code> object with the passed keys and environs
	 * 
	 * @param keys
	 * @param environs
	 */
	public World(String[] keys, Environment[] environs) {
		this();
		// TODO: fill this in
	}

	/**
	 * Gets an <code>Environment</code> by identifier
	 * 
	 * @param key
	 *            The identifier of the <code>Environment</code> desired
	 * @return The <code>Environment</code> requested
	 */
	public Environment getEnvironment(String key) {
		return (Environment) this.locate(key);
	}

	/**
	 * Gets the <code>World</code>'s active <code>Environment</code>
	 * 
	 * @return The current <code>Environment</code>
	 */
	public Environment getCurrentEnvironment() {
		return (Environment) this.getInventoryItem("Current");
	}

	/**
	 * Sets the current <code>Environment</code> to an <code>Environment</code> specified by the identifier
	 * 
	 * @param key
	 *            An identifier of a stored <code>Environment</code>
	 */
	public void setCurrentEnvironment(String key) {
		this.addInventoryItem("Current", this.getInventoryItem(key));
	}

	/**
	 * Sets the current <code>Environment</code> to the <code>Environment</code> specified
	 * 
	 * @param e
	 *            The <code>Environment</code> to set as current
	 */
	public void setCurrentEnvironment(Environment e) {
		this.addInventoryItem("Current", e);
	}

	/**
	 * Adds an <code>Environment</code> or a container to the <code>World</code>
	 * 
	 * @param meta
	 *            The <code>Environment</code> or container to add
	 * @return The added object (for chaining)
	 */
	public MetaObject add(String key, MetaObject meta) {
		this.addInventoryItem(key, meta);
		return (MetaObject) this.getInventoryItem(key);
	}

	/**
	 * Adds an <code>Environment</code> or a container to the <code>World</code>
	 * 
	 * @return The added object (for chaining)
	 */
	public MetaObject add(String key) {
		this.addInventoryItem(key, new MetaAdapter());
		return (MetaObject) this.getInventoryItem(key);
	}

	/**
	 * Checks to see if an object specified by an identifier is in the world
	 * 
	 * @param key
	 *            The identifier
	 * @return true if object is found, false otherwise
	 */
	public boolean isInWorld(String key) {
		if (this.isEmpty(key))
			return false;
		return true;
	}

}
